@@ -2,12 +2,16 @@ import array
 import asyncio
 import logging
 import socket
+import sys
 import traceback
 from collections import deque
 from copy import copy
 from typing import Any, Optional
 
-import async_timeout
+if sys.version_info[:2] < (3, 11):
+    from async_timeout import timeout as asyncio_timeout
+else:
+    from asyncio import timeout as asyncio_timeout
 
 from .. import introspection as intr
 from .._private.unmarshaller import Unmarshaller
@@ -166,7 +170,6 @@ class MessageBus(BaseMessageBus):
         super().__init__(bus_address, bus_type, ProxyObject)
         self._negotiate_unix_fd = negotiate_unix_fd
         self._loop = asyncio.get_running_loop()
-        self._unmarshaller = self._create_unmarshaller()
 
         self._writer = _MessageWriter(self)
 
@@ -197,7 +200,8 @@ class MessageBus(BaseMessageBus):
         self._loop.add_reader(
             self._fd,
             build_message_reader(
-                self._unmarshaller,
+                self._stream,
+                self._sock if self._negotiate_unix_fd else None,
                 self._process_message,
                 self._finalize,
             ),
@@ -268,7 +272,7 @@ class MessageBus(BaseMessageBus):
 
         super().introspect(bus_name, path, reply_handler)
 
-        async with async_timeout.timeout(timeout):
+        async with asyncio_timeout(timeout):
             return await future
 
     async def request_name(
@@ -430,7 +434,9 @@ class MessageBus(BaseMessageBus):
     async def _auth_readline(self) -> str:
         buf = b""
         while buf[-2:] != b"\r\n":
-            buf += await self._loop.sock_recv(self._sock, 2)
+            # The auth protocol is line based, so we can read until we get a
+            # newline.
+            buf += await self._loop.sock_recv(self._sock, 1024)
         return buf[:-2].decode()
 
     async def _authenticate(self) -> None:
@@ -455,6 +461,9 @@ class MessageBus(BaseMessageBus):
                 )
                 self._stream.flush()
             if response == "BEGIN":
+                # The first octet received by the server after the \r\n of the BEGIN command
+                # from the client must be the first octet of the authenticated/encrypted stream
+                # of D-Bus messages.
                 break
 
     def disconnect(self) -> None:
@@ -468,12 +477,6 @@ class MessageBus(BaseMessageBus):
         except Exception:
             logging.warning("could not close socket", exc_info=True)
 
-    def _create_unmarshaller(self) -> Unmarshaller:
-        sock = None
-        if self._negotiate_unix_fd:
-            sock = self._sock
-        return Unmarshaller(self._stream, sock)
-
     def _finalize(self, err: Optional[Exception] = None) -> None:
         try:
             self._loop.remove_reader(self._fd)
